AvoVPAIDWrapper = function() {
    this._creative = getVPAIDAd();
    this._ui = getUI();
    this.timer = null;
    
    AvoVPAIDWrapper.prototype.setVpaidDelegate = function(vpaidDelegate) {
        this._vpaidDelegate = vpaidDelegate;
    }

    AvoVPAIDWrapper.prototype.setVpaidCallback = function(vpaidCallback) {
         this._vpaidCallback = vpaidCallback;
    }

    AvoVPAIDWrapper.prototype.checkVPAIDInterface = function() {
        if(this._ad.handshakeVersion && typeof this._ad.handshakeVersion == "function"
            && this._ad.initAd && typeof this._ad.initAd == "function"
            && this._ad.startAd && typeof this._ad.startAd == "function"
    		&& this._ad.stopAd && typeof this._ad.stopAd == "function"
    		&& this._ad.skipAd && typeof this._ad.skipAd == "function"
    		&& this._ad.resizeAd && typeof this._ad.resizeAd == "function"
    		&& this._ad.pauseAd && typeof this._ad.pauseAd == "function"
    		&& this._ad.resumeAd && typeof this._ad.resumeAd == "function"
    		&& this._ad.expandAd && typeof this._ad.expandAd == "function"
    		&& this._ad.collapseAd && typeof this._ad.collapseAd == "function"
    		&& this._ad.subscribe && typeof this._ad.subscribe == "function"
    		&& this._ad.unsubscribe && typeof this._ad.unsubscribe == "function") {
            return true;
    	}
    	return false;
    };
    
    AvoVPAIDWrapper.prototype.handshakeVersion = function(version) {
        console.log("VPAID bridge log: handshakeVersion(version: " + version + ")");
        var version = this._creative.handshakeVersion(version);
        return version;
    }

    AvoVPAIDWrapper.prototype.initAd = function(width, height, viewMode, desiredBitrate, creativeData, environmentVars) {

        console.log("VPAID bridge log: initAd(width: " + width + ", height: " + height
        + ", viewMode: " + viewMode + ", desiredBitrate: " + desiredBitrate + ", creativeData: "
        + creativeData + ", environmentVars: " + environmentVars + ")");
        this._ui.resize(width, height);
        this._creative.initAd(width, height, viewMode, desiredBitrate, creativeData, environmentVars);
        this._vpaidDelegate.onAdInited();
    };
    
    AvoVPAIDWrapper.prototype.onAdPaused = function() {
        console.log("VPAID bridge log: onAdPaused");
        var remainingTime = this._creative.getAdRemainingTime();
        var duration = this._creative.getAdDuration();
        this._vpaidDelegate.onAdPaused(remainingTime, duration);
    };
    
    AvoVPAIDWrapper.prototype.onAdPlaying = function() {
        console.log("VPAID bridge log: onAdPlaying");
        var remainingTime = this._creative.getAdRemainingTime();
        var duration = this._creative.getAdDuration();
        this._vpaidDelegate.onAdPlaying(remainingTime, duration);
    };
    
    AvoVPAIDWrapper.prototype.onAdError = function(message) {
        console.log("VPAID bridge log: onAdError: " + message);
        this._vpaidDelegate.onAdError(message);
    };
    
    AvoVPAIDWrapper.prototype.onAdLog = function(message) {
        console.log("VPAID bridge log: onAdLog: " + message);
        this._vpaidDelegate.onAdLog(message);
    };
    
    AvoVPAIDWrapper.prototype.onAdUserAcceptInvitation = function() {
        console.log("VPAID bridge log: onAdUserAcceptInvitation");
        this._vpaidDelegate.onAdUserAcceptInvitation();
    };
    
    AvoVPAIDWrapper.prototype.onAdUserMinimize = function() {
        console.log("VPAID bridge log: onAdUserMinimize");
        this._vpaidDelegate.onAdUserMinimize();
    };
    
    AvoVPAIDWrapper.prototype.onAdUserClose = function() {
        console.log("VPAID bridge log: onAdUserClose");
        this._vpaidDelegate.onAdUserClose();
    };
    
    AvoVPAIDWrapper.prototype.onAdSkippableStateChange = function() {
        var skippable = this._creative.getAdSkippableState();
        console.log("VPAID bridge log: Ad Skippable State Changed to: " + skippable);
        this._vpaidDelegate.onAdSkippableStateChange(skippable);
    };
    
    AvoVPAIDWrapper.prototype.onAdExpandedChange = function() {
        var expanded = this._creative.getAdExpanded();
        console.log("VPAID bridge log: Ad Expanded Changed to: " + expanded);
        this._vpaidDelegate.onAdExpandedChange(expanded);
    };
    
    AvoVPAIDWrapper.prototype.getAdExpanded = function() {
        console.log("VPAID bridge log: getAdExpanded");
        var expanded = this._creative.getAdExpanded();
        if (this._vpaidCallback != null) {
            this._vpaidCallback.onGetAdExpandedResult(expanded);
        }
        return expanded;
    };

    AvoVPAIDWrapper.prototype.getHandshakeVersion = function(version) {
    console.log("VPAID bridge log: getHandshakeVersion");
        var version = this._creative.handshakeVersion(version);
        if (this._vpaidCallback != null) {
            this._vpaidCallback.onHandshakeVersionResult(version);
        }
    }

    AvoVPAIDWrapper.prototype.getAdSkippableState = function() {
        console.log("VPAID bridge log: getAdSkippableState");
        var skippable = this._creative.getAdSkippableState();
        if (this._vpaidCallback != null) {
            this._vpaidCallback.onGetAdSkippableStateResult(skippable);
        }
        return skippable;
    };

    AvoVPAIDWrapper.prototype.onAdSizeChange = function() {
        var width = this._creative.getAdWidth();
        var height = this._creative.getAdHeight()
        console.log("VPAID bridge log: Ad size changed: width = " + width + "; height = " + height);
        this._vpaidDelegate.onAdSizeChange(width, height);
    };

    AvoVPAIDWrapper.prototype.onAdDurationChange = function() {
        var duration = this._creative.getAdDuration();
        console.log("VPAID bridge log: onAdDurationChange: duration = " + duration);
        if (this.handshakeVersion() >= 2) {
            this._ui.moveProgress(this._creative.getAdRemainingTime(), duration);
        }
        this._vpaidDelegate.onAdDurationChange(duration);
    };

    AvoVPAIDWrapper.prototype.onAdRemainingTimeChange = function() {
        var remainingTime = this._creative.getAdRemainingTime();
        console.log("VPAID bridge log: onAdRemainingTimeChange: remainingTime = " + remainingTime);
        if (this.handshakeVersion() < 2) {
            this._ui.moveProgress(remainingTime, this._creative.getAdDuration());
        }
        this._vpaidDelegate.onAdRemainingTimeChange(remainingTime);
    };

    AvoVPAIDWrapper.prototype.getAdRemainingTime = function() {
        console.log("VPAID bridge log: getAdRemainingTime");
        var remainingTime = this._creative.getAdRemainingTime();
        if (this._vpaidCallback != null) {
            this._vpaidCallback.onGetAdRemainingTimeResult(remainingTime);
        }
        return remainingTime;
    };

    AvoVPAIDWrapper.prototype.getAdDuration = function() {
        console.log("VPAID bridge log: getAdDuration");
        var duration = this._creative.getAdDuration();
        if (this._vpaidCallback != null) {
            this._vpaidCallback.onGetAdDurationResult(duration);
        }
        return duration;
    };

    AvoVPAIDWrapper.prototype.onAdImpression = function() {
        console.log("VPAID bridge log: Ad Impression");
        var remainingTime = this._creative.getAdRemainingTime();
        var duration = this._creative.getAdDuration();
        this._vpaidDelegate.onAdImpression(remainingTime, duration);
    };

    AvoVPAIDWrapper.prototype.onAdClickThrough = function(url, id, playerHandles) {
        console.log("VPAID bridge log: onAdClickThrough: " + url );
        var adjustedUrl = url;
        if (adjustedUrl == undefined) {
            adjustedUrl = ""
        }
        var remainingTime = this._creative.getAdRemainingTime();
        var duration = this._creative.getAdDuration();
        this._vpaidDelegate.onAdClickThrough(adjustedUrl, id, playerHandles, remainingTime, duration);
    };

    AvoVPAIDWrapper.prototype.onAdInteraction = function(id) {
        console.log("VPAID bridge log: onAdInteraction");
        this._vpaidDelegate.onAdInteraction(id);
    };

    AvoVPAIDWrapper.prototype.onAdVideoStart = function() {
        console.log("VPAID bridge log: Video has started");
        var remainingTime = this._creative.getAdRemainingTime();
        var duration = this._creative.getAdDuration();
        this._vpaidDelegate.onAdVideoStart(remainingTime, duration);
    };

    AvoVPAIDWrapper.prototype.onAdVideoFirstQuartile = function() {
        console.log("VPAID bridge log: Video 25% completed");
        var remainingTime = this._creative.getAdRemainingTime();
        var duration = this._creative.getAdDuration();
        this._vpaidDelegate.onAdVideoFirstQuartile(remainingTime, duration);
    };

    AvoVPAIDWrapper.prototype.onAdVideoMidpoint = function() {
        console.log("VPAID bridge log: Video 50% completed");
        var remainingTime = this._creative.getAdRemainingTime();
        var duration = this._creative.getAdDuration();
        this._vpaidDelegate.onAdVideoMidpoint(remainingTime, duration);
    };

    AvoVPAIDWrapper.prototype.onAdVideoThirdQuartile = function() {
        console.log("VPAID bridge log: Video 75% completed");
        var remainingTime = this._creative.getAdRemainingTime();
        var duration = this._creative.getAdDuration();
        this._vpaidDelegate.onAdVideoThirdQuartile(remainingTime, duration);
    };

    AvoVPAIDWrapper.prototype.onAdVideoComplete = function() {
        console.log("VPAID bridge log: Video 100% completed");
        var remainingTime = this._creative.getAdRemainingTime();
        var duration = this._creative.getAdDuration();
        this._vpaidDelegate.onAdVideoComplete(remainingTime, duration);
    };

    AvoVPAIDWrapper.prototype.onAdLinearChange = function() {
        var adLinear = this._creative.getAdLinear();
        console.log("VPAID bridge log: Ad linear has changed: " + adLinear);
        this._vpaidDelegate.onAdLinearChange(adLinear);
    };

    AvoVPAIDWrapper.prototype.getAdLinear = function() {
        var adLinear = this._creative.getAdLinear();
        console.log("VPAID bridge log: getAdLinear");
        if (this._vpaidCallback != null) {
            this._vpaidCallback.onGetAdLinearResult(adLinear);
        }
        return adLinear;
    };

    AvoVPAIDWrapper.prototype.onAdLoaded = function() {
        console.log("VPAID bridge log: Ad has been loaded");
        this._vpaidDelegate.onAdLoaded();
    };

    AvoVPAIDWrapper.prototype.onAdStarted = function() {
        console.log("VPAID bridge log: Ad has started");
        this.timer = setInterval(function() {
            var remainingTime = this._creative.getAdRemainingTime();
            var duration = this._creative.getAdDuration();
            if (remainingTime >= 0 && duration >= 0) {
                this._ui.moveProgress(remainingTime, duration);
            }
        }.bind(this), 200);
        this._vpaidDelegate.onAdStarted(this._creative.getAdRemainingTime(), this._creative.getAdDuration());
    };

    AvoVPAIDWrapper.prototype.onAdStopped = function() {
        console.log("VPAID bridge log: Ad has stopped");
        var remainingTime = this._creative.getAdRemainingTime();
        var duration = this._creative.getAdDuration();
        clearInterval(this.timer);
        this._vpaidDelegate.onAdStopped(remainingTime, duration);
    };

    AvoVPAIDWrapper.prototype.onAdSkipped = function() {
        console.log("VPAID bridge log: Ad was skipped");
        var remainingTime = this._creative.getAdRemainingTime();
        var duration = this._creative.getAdDuration();
        this._creative.stopAd();
        this._vpaidDelegate.onAdSkipped(remainingTime, duration);
    };

    AvoVPAIDWrapper.prototype.setAdVolume = function(volume) {
        console.log("VPAID bridge log: setAdVolume: " + volume);
        this._creative.setAdVolume(volume);
    };

    AvoVPAIDWrapper.prototype.getAdVolume = function() {
        var volume = this._creative.getAdVolume();
        console.log("VPAID bridge log: getAdVolume");
        if (this._vpaidCallback != null) {
            this._vpaidCallback.onGetAdVolumeResult(volume);
        }
        return volume;
    };

    AvoVPAIDWrapper.prototype.onAdVolumeChange = function() {
        var volume = this._creative.getAdVolume();
        var remainingTime = this._creative.getAdRemainingTime();
        var duration = this._creative.getAdDuration();
        console.log("Ad Volume has changed to: " + volume);
        this._vpaidDelegate.onAdVolumeChanged(volume, remainingTime, duration);
    };

    AvoVPAIDWrapper.prototype.startAd = function() {
        console.log("VPAID bridge log: startAd");
        this._creative.startAd();
        this._ui.createProgressBar();
    };

    AvoVPAIDWrapper.prototype.skipAd = function() {
        console.log("VPAID bridge log: skipAd");
        var remainingTime = this._creative.getAdRemainingTime();
        var duration = this._creative.getAdDuration();
        this._creative.skipAd(remainingTime, duration);
    };

    AvoVPAIDWrapper.prototype.stopAd = function() {
        console.log("VPAID bridge log: stopAd");
        this._creative.stopAd();
    };

    AvoVPAIDWrapper.prototype.resizeAd = function(width, height, viewMode) {
        console.log("VPAID bridge log: resizeAd");
        this._ui.resize(width, height);
        this._creative.resizeAd(width, height, viewMode);
    };

    AvoVPAIDWrapper.prototype.pauseAd = function() {
        console.log("VPAID bridge log: pauseAd");
        this._creative.pauseAd();
    };

    AvoVPAIDWrapper.prototype.resumeAd = function() {
        console.log("VPAID bridge log: resumeAd");
        this._creative.resumeAd();
    };

    AvoVPAIDWrapper.prototype.expandAd = function() {
        console.log("VPAID bridge log: expandAd");
        this._creative.expandAd();
    };

    AvoVPAIDWrapper.prototype.collapseAd = function() {
        console.log("VPAID bridge log: collapseAd");
        this._creative.collapseAd();
    };

    AvoVPAIDWrapper.prototype.onAdSkipPress = function() {
    	console.log("VPAID bridge log:onAdSkipPress");
    	this._creative.skipAd();
    }

    AvoVPAIDWrapper.prototype.setCallbacksForCreative = function() {
        var callbacks = {
            'AdStarted': this.onAdStarted,
            'AdStopped': this.onAdStopped,
            'AdSkipped': this.onAdSkipped,
            'AdLoaded': this.onAdLoaded,
            'AdLinearChange': this.onAdLinearChange,
            'AdSizeChange': this.onAdSizeChange,
            'AdExpandedChange': this.onAdExpandedChange,
            'AdSkippableStateChange': this.onAdSkippableStateChange,
            'AdDurationChange': this.onAdDurationChange,
            'AdRemainingTimeChange': this.onAdRemainingTimeChange,
            'AdVolumeChange': this.onAdVolumeChange,
            'AdImpression': this.onAdImpression,
            'AdClickThru': this.onAdClickThrough,
            'AdInteraction': this.onAdInteraction,
            'AdVideoStart': this.onAdVideoStart,
            'AdVideoFirstQuartile': this.onAdVideoFirstQuartile,
            'AdVideoMidpoint': this.onAdVideoMidpoint,
            'AdVideoThirdQuartile': this.onAdVideoThirdQuartile,
            'AdVideoComplete': this.onAdVideoComplete,
            'AdUserAcceptInvitation': this.onAdUserAcceptInvitation,
            'AdUserMinimize': this.onAdUserMinimize,
            'AdUserClose': this.onAdUserClose,
            'AdPaused': this.onAdPaused,
            'AdPlaying': this.onAdPlaying,
            'AdError': this.onAdError,
            'AdLog': this.onAdLog
        };
        for (var eventName in callbacks) {
            this._creative.subscribe(callbacks[eventName], eventName, this);
        }
    };

    AvoVPAIDWrapper.prototype.setCallbacksForUI = function() {
        var callbacks = {
            'AdSkipped': this.onAdSkipPress,
        };
        for (var eventName in callbacks) {
            this._ui.subscribe(callbacks[eventName], eventName, this);
        }
    };
}

initVpaidWrapper = function() {
    avoVPAIDWrapperInstance = new AvoVPAIDWrapper();
    avoVPAIDWrapperInstance.setCallbacksForCreative();
    avoVPAIDWrapperInstance.setVpaidDelegate(Android);
    avoVPAIDWrapperInstance.setVpaidCallback(Android);
    avoVPAIDWrapperInstance.setCallbacksForUI()
    Android.onWrapperReady();
}