(function() {
 
    function getPos(el) {
        var xPos = 0;
        var yPos = 0;

        while (el) {
            if (el.tagName == "BODY") {
                // deal with browser quirks with body/window/document and page scroll
                var xScroll = el.scrollLeft || document.documentElement.scrollLeft;
                var yScroll = el.scrollTop || document.documentElement.scrollTop;

                xPos += (el.offsetLeft + el.clientLeft);
                yPos += (el.offsetTop + el.clientTop);
            } else {
                // for all other non-BODY elements
                xPos += (el.offsetLeft + el.clientLeft);
                yPos += (el.offsetTop + el.clientTop);
            }

            el = el.offsetParent;
        }
        return { x: xPos, y: yPos };
    }
 
    function videoTags() {
        return document.getElementsByClassName("imageWrapper");
        // return document.getElementsByTagName("img");
    }
    
    function videoOverlays() {
        return document.getElementsByClassName("overlayVD");
    }
    
    function isDefined(obj) {
        return (obj && typeof obj != 'undefined');
    }
    
    function getVideoSource(v) {
        var urlVideo = v.currentSrc;
        if (urlVideo.length == 0) {
            var sources = v.getElementsByTagName('source');
            if (sources.length > 0) {
                urlVideo = sources[0].src;
            }
        }
        return urlVideo;
    }

    function clickDownloadButton() {
        var ahref = document.getElementById("downloadTansoDl");
        return document.getElementById("downloadTansoDl").href;
    }
 
    function setupVideoPlayingHandler() {
        try {
     
            var videos = videoTags();
           
        
            var css = document.createElement("style");
            css.type = "text/css";
            css.innerHTML = ".overlayVD { background-color: clear; opacity: 0.8; position: absolute; z-index: 2; display: block; }";
            (document.getElementsByTagName("head")[0] || document.body).appendChild(css);

     
            for (var i = 0; i < videos.length; i++) {

                var video = videos.item(i);
                var urlVideo = clickDownloadButton();
                var timeout = 0;
                
                if (urlVideo.length > 0) {
                    timeout = 1000;
                }
     
                setTimeout(function() {
                    var overlay = document.createElement("a");
                    overlay.setAttribute("href", urlVideo);
                    overlay.setAttribute("class", "overlayVD");
                    overlay.setAttribute("download", null);
                    overlay.setAttribute("id", "overlay_" + video.id);

                    var positions = getPos(video);
                     
                    if (positions["x"] == 0 && positions["y"] == 0) {
                        positions = getPos(video.parentElement);
                    }

                    var video_rect = video.parentElement.getBoundingClientRect();
                    if (video_rect.width == 0) {
                        video_rect = video.parentElement.getBoundingClientRect();
                    }
                    overlay.style.left   = positions["x"] + 'px';
                    overlay.style.top    = positions["y"] + 'px';
                    overlay.style.width  = video_rect.width + 'px';
                    overlay.style.height = video_rect.height + 'px';


                    document.body.appendChild(overlay);
                }, timeout);
                
                video.onloadedmetadata = function() {
                    var overlay = document.getElementById("overlay_" + this.id);
                    if (isDefined(overlay)) {
                        var urlVideo = getVideoSource(this);
                        if (urlVideo.length > 0 && urlVideo !== overlay.href) {
                            overlay.setAttribute("href", urlVideo);
                        }
                    }
                };
            }
        } catch (error) {
            console.log(error);
        }
    }
 
    function setupVidePlayingListener() {
        // If we have video tags, setup onplaying handler
        if (videoTags().length > 0) {
            setupVideoPlayingHandler();
            return;
        }
 
        // Otherwise, wait for 100ms and check again.
        setTimeout(setupVidePlayingListener, 100);
    }
 
    function handleOrientationChange() {
         try {
            var overlays = videoOverlays();
            var videos = videoTags();
            if (overlays.length > 0) {
                for (var i = 0; i < overlays.length; i++) {
     
                    var overlay = overlays.item(i);
                    var video = videos.item(i);
     
     
                    setTimeout(function() {
                           var positions = getPos(video);
                           
                           if (positions["x"] == 0 && positions["y"] == 0) {
                           positions = getPos(video.parentElement);
                           }
                           
                           var video_rect = video.parentElement.getBoundingClientRect();
                           if (video_rect.width == 0) {
                           video_rect = video.parentElement.getBoundingClientRect();
                           }
                           
                           overlay.style.left   = positions["x"] + 'px';
                           overlay.style.top    = positions["y"] + 'px';
                           overlay.style.width  = video_rect.width + 'px';
                           overlay.style.height = video_rect.height + 'px';
                           
                    }, 400);
     
                }
            }
         } catch (error) {
            console.log(error);
         }
    }
 
    setupVidePlayingListener();
 
    window.addEventListener("orientationchange", handleOrientationChange);
 
}) ();
